// Client.cpp: implementation of the CClient class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "../../otherfunctions.h"
#include "Client.h"


//////////////////////////////////////////////////////////////////////
CClient::CClient()
	:m_pParent(NULL)
	,m_bConnected(false)
	,m_uCountry(0)
	,m_sFullIP(_T(""))
	,m_sUserName(_T(""))
	,m_pReqFile(NULL)
{
	m_Addr.Addr = 0;
	m_Addr.Port = 0;
}

//////////////////////////////////////////////////////////////////////
CClient::~CClient()
{
}

//////////////////////////////////////////////////////////////////////
bool CClient::operator==(const AddrPort Addr)
{
	return (m_Addr.Addr == Addr.Addr && m_Addr.Port == Addr.Port);
}

//////////////////////////////////////////////////////////////////////
CClientMule::CClientMule(AddrPort Addr, HashType Hash)
{
	m_Addr.Addr = Addr.Addr;
	m_Addr.Port = Addr.Port;
	md4cpy(&m_Hash, &Hash);
	m_eClientSoft = SO_UNKNOWN;
}

//////////////////////////////////////////////////////////////////////
CClientMule::~CClientMule()
{
}

//////////////////////////////////////////////////////////////////////
bool CClientMule::operator==(const CClient* pClient)
{
	return false;
}

//////////////////////////////////////////////////////////////////////
bool CClientMule::operator==(const HashType Hash)
{
	return !md4cmp(&m_Hash, &Hash);
}

//////////////////////////////////////////////////////////////////////
void CClientMule::_PutMuleVersion(BYTE nMuleVersion)
{
	m_dwClientVersion = nMuleVersion;
	m_nCompatibleClient = (m_dwClientVersion >> 24);
	m_dwClientVersion &= 0x00FFFFFF;
	m_nEmuleVersion = 0x99;
//	m_fSupportsAskSharedDirs = 1;
}

//////////////////////////////////////////////////////////////////////
EnumClientTypes CClientMule::GetHashType()
{
	if (m_Hash.hash[5] == 14 && m_Hash.hash[14] == 111)
		return SO_EMULE;
	else if (m_Hash.hash[5] == 13 && m_Hash.hash[14] == 110)
		return SO_OLDEMULE;
	else if (m_Hash.hash[5] == 'M' && m_Hash.hash[14] == 'L')
		return SO_MLDONKEY;
	else
		return SO_UNKNOWN;
}

//////////////////////////////////////////////////////////////////////
void CClientMule::OnConnected()
{
	CClient::OnConnected();
}

//////////////////////////////////////////////////////////////////////
void CClientMule::OnDisconnected()
{
	CClient::OnDisconnected();
	// change state, take care of upload/download statuses, etc
}

//////////////////////////////////////////////////////////////////////
void CClientMule::RequestFile(CKnownFile* pFile)
{
	m_pReqFile = pFile;
	m_tmLastUpRequest = CPreciseTime::GetCurrentTime();

	// Add client to waiting queue
	g_stEngine.ClientList.AddToWaitingQueue(this);

	// Check requests, etc - CUpDownClient::AddRequestCount
}


//////////////////////////////////////////////////////////////////////
// ClientList
CClientList::CClientList()
{
}

//////////////////////////////////////////////////////////////////////
CClientList::~CClientList()
{
	for(ClientMap::iterator it = m_Clients.begin(); it != m_Clients.end(); it++)
	{
		CClient* pClient = (*it).first;
		if(pClient)
			delete pClient;
	}
	m_Clients.erase(m_Clients.begin(), m_Clients.end());
}

//////////////////////////////////////////////////////////////////////
void CClientList::AddClient(CClient* pClient)
{
	m_Clients[pClient] = true;
}

//////////////////////////////////////////////////////////////////////
void CClientList::RemoveClient(CClient* pClient)
{
}

//////////////////////////////////////////////////////////////////////
CClientMule* CClientList::FindMuleClient(AddrPort Addr)
{
	for(ClientMap::iterator it = m_Clients.begin(); it != m_Clients.end(); it++)
	{
		CClient* pClient = (*it).first;
		if(pClient && pClient->Type == CLIENT_MULE)
		{
			if(*pClient == Addr)
				return reinterpret_cast<CClientMule*>(pClient);
		}
	}
	return NULL;
}

//////////////////////////////////////////////////////////////////////
CClientMule* CClientList::FindMuleClient(HashType Hash)
{
	for(ClientMap::iterator it = m_Clients.begin(); it != m_Clients.end(); it++)
	{
		CClient* pClient = (*it).first;
		if(pClient && pClient->Type == CLIENT_MULE)
		{
			CClientMule* pMule = reinterpret_cast<CClientMule*>(pClient);
			if(*pMule == Hash)
				return pMule;
		}
	}
	return NULL;
}

//////////////////////////////////////////////////////////////////////
bool CClientList::IsValidClient(CClient* pClient)
{
	return (m_Clients.find(pClient) != m_Clients.end());
}

//////////////////////////////////////////////////////////////////////
void CClientList::AddToWaitingQueue(CClient* pClient)
{
	if(pClient && pClient->Type == CLIENT_MULE)
	{
		CClientMule* pMule = reinterpret_cast<CClientMule*>(pClient);

		// Filtering invalid eMule clients
		if(pMule->MuleVersion == 0 && pMule->ClientVersion == 0 &&
			(pMule->ClientSoft == SO_EMULE || pMule->ClientSoft == SO_PLUS || pMule->ClientSoft == SO_OLDEMULE))
		{
			return;
		}

		m_Waiting[pClient] = true;
	}
}

//////////////////////////////////////////////////////////////////////
bool CClientList::RemoveFromWaitingQueue(CClient* pClient)
{
	if(pClient == NULL)
		return false;
	ClientMap::iterator it = m_Waiting.find(pClient);
	if(it != m_Waiting.end())
	{
		m_Waiting.erase(it);
		return true;
	}
	return false;
}
