// Client.h: interface for the CClient class.
//
//////////////////////////////////////////////////////////////////////

#include "../../Loggable.h"
#include "../../OtherFunctions.h"

enum EnumClientType
{
	CLIENT_MULE
};

struct CEmClient_Peer;

class CClient : public CLoggable
{
public:
	CClient();
	virtual ~CClient();

	// Basics
	__declspec(property(get=_GetClientType)) EnumClientType Type;
	__declspec(property(get=_GetParent, put=_PutParent))	CEmClient_Peer*	Parent;

	virtual EnumClientType _GetClientType() = 0;
	CEmClient_Peer* _GetParent(){ return m_pParent; }
	void _PutParent(CEmClient_Peer* pParent){ m_pParent = pParent; }

	// Comparison
	virtual bool operator==(const CClient* pClient) = 0;
	virtual bool operator==(const AddrPort Addr);

	// Connection state
	virtual void OnConnected(){ m_bConnected = true; }
	virtual void OnDisconnected(){ m_bConnected = false; }

	__declspec(property(get=_GetConnected)) bool Connected;

	bool _GetConnected() const { return m_bConnected; }

	// Identification
	__declspec(property(get=_GetUserName, put=_PutUserName)) CString UserName;

	CString	_GetUserName() const { return m_sUserName; }
	void _PutUserName(CString sUserName) { m_sUserName = sUserName; }

	USHORT	ClientCountry(){ return m_uCountry; }

	// Upload
	__declspec(property(get=_GetUpState, put=_PutUpState))	EnumULQState	UploadState;
	__declspec(property(get=_GetReqFile))					CKnownFile*		ReqFile;
	__declspec(property(get=_GetLastUpReq))					CPreciseTime	LastUploadRequest;

	EnumULQState _GetUpState() const { return m_eUploadState; }
	CKnownFile* _GetReqFile() const { return m_pReqFile; }
	CPreciseTime _GetLastUpReq() const { return m_tmLastUpRequest; }

	void _PutUpState(EnumULQState eUploadState){ m_eUploadState = eUploadState; }

protected:
	// Basics
	CEmClient_Peer*	m_pParent;

	// Connection data
	AddrPort	m_Addr;
	CString		m_sFullIP;
	bool		m_bConnected;

	// Identification
	CString		m_sUserName;
	USHORT		m_uCountry;

	// Upload
	EnumULQState	m_eUploadState;
	CKnownFile*		m_pReqFile;
	CPreciseTime	m_tmLastUpRequest;
};

class CClientMule : public CClient
{
public:
	CClientMule(AddrPort Addr, HashType Hash);
	virtual ~CClientMule();

	virtual EnumClientType _GetClientType(){ return CLIENT_MULE; };

	// Comparison
	virtual bool operator==(const CClient* pClient);
	virtual bool operator==(const HashType Hash);

	// Connection state
	virtual void OnConnected();
	virtual void OnDisconnected();

	EnumClientTypes GetHashType();

	// Connection data, identification
	__declspec(property(put=_PutClientPort))	USHORT	ClientPort;
	__declspec(property(put=_PutClientUDPPort))	USHORT	ClientUDPPort;
	__declspec(property(get=_GetClientVersion, put=_PutClientVersion))	DWORD	ClientVersion;
	__declspec(property(get=_GetMuleVersion, put=_PutMuleVersion))		BYTE	MuleVersion;
	__declspec(property(get=_GetClientSoft))	EnumClientTypes	ClientSoft;

	DWORD	_GetClientVersion() const { return m_dwClientVersion; }
	BYTE	_GetMuleVersion() const { return m_nEmuleVersion; }
	EnumClientTypes _GetClientSoft() const { return m_eClientSoft; }

	void _PutClientPort(USHORT nPort){ m_Addr.Port = nPort; }
	void _PutClientUDPPort(USHORT nUDPPort){ m_nUDPPort = nUDPPort; }
	void _PutClientVersion(DWORD dwClientVersion){ m_dwClientVersion = dwClientVersion; }
	void _PutMuleVersion(BYTE nMuleVersion);

	// Uploading
	void RequestFile(CKnownFile* pFile);

private:
	// Additional connection data
	USHORT	m_nUDPPort;

	// Come from server
	AddrPort	m_ServerAddr;

	// Additional identification
	DWORD		m_dwUserID;
	HashType	m_Hash;

	// Software identification
	EnumClientTypes	m_eClientSoft;
	CString	m_sClientSoft;
	DWORD	m_dwClientVersion;
	DWORD	m_dwPlusVers;
	BYTE	m_nEmuleVersion;
	BYTE	m_nCompatibleClient;
	bool	m_bIsHybrid;
	bool	m_bIsMLDonkey;

	// Supported protocols & features
	bool	m_bIsLowID;
	bool	m_bEmuleProtocol;
	BYTE	m_nAcceptCommentVer;
	BYTE	m_nDataCompVer;
	BYTE	m_nExtendedRequestsVer;
	BYTE	m_nSourceExchangeVer;
	BYTE	m_nUDPVer;
	BYTE	m_nSupportSecIdent;
};

typedef map<CClient*, bool> ClientMap;

class CClientList : public CLoggable
{
public:
	CClientList();
	virtual ~CClientList();

	void AddClient(CClient* pClient);
	void RemoveClient(CClient* pClient);
	CClientMule* FindMuleClient(AddrPort Addr);
	CClientMule* FindMuleClient(HashType Hash);
	bool IsValidClient(CClient* pClient);

	void AddToWaitingQueue(CClient* pClient);
	bool RemoveFromWaitingQueue(CClient* pClient);

private:
	ClientMap m_Clients;
	ClientMap m_Waiting;
};
