// EmEngine.h: interface for the CEmEngine class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EMENGINE_H__F77B635A_251D_4554_9D96_DACFCA968759__INCLUDED_)
#define AFX_EMENGINE_H__F77B635A_251D_4554_9D96_DACFCA968759__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../Loggable.h"

class CServer;
struct CTask;
struct CEmClient;
struct CEmClient_Server;
struct CTcpCompletionTask;
class COpCode;
class CTaskProcessor_Main;
class CTaskProcessor_Sockets;
class CTaskProcessor_Files;
class CTaskProcessor_Logger;
class CTaskProcessor_DB;
class CPreferences;
class CClientList;

enum EnumServerStates
{
	SERVER_DISCONNECTED,
	SERVER_CONNECTING,
	SERVER_WAITFORLOGIN,
	SERVER_CONNECTED
};

struct ServerState_Struct
{
	SOCKET		hSocket;
	ULONG		nAddr;
	USHORT		nPort;
	short		nConnState;
	long		nClientID;

	bool IsConnecting()
	{
		return (nConnState == SERVER_CONNECTING	|| nConnState == SERVER_WAITFORLOGIN);
	}
	bool IsConnected()
	{ 
		return (nConnState == SERVER_CONNECTED);
	}
	bool IsLowID()
	{ 
		return (nClientID < 0x1000000); 
	}
};

class CEmEngine : public CLoggable
{
	HANDLE m_hStop; // termination event

public:

	CEmEngine();
	~CEmEngine() { Uninit(); }

	bool Init();
	void Uninit();

	// Public methods
	void ConnectToServer(CString sAddr = "", ULONG ulPort = 0);
	void ConnectToAnyServer();
	void DisconnectFromServer(SOCKET hPrevServer = NULL);

	// Data
	SOCKET GetServerSocket(){ return m_stServer.hSocket; }
	long GetClientID(){ return m_stServer.nClientID; }
	bool IsLocalServer(ULONG nAddr, USHORT uPort);

	// Update Server Info
	void SetConnectionState(short nConnectionState);
	void SetClientID(long cID);
	void ConnectedTo(CEmClient_Server* pClient);

	inline bool IsShuttingDown() 
	{
		return WAIT_TIMEOUT != WaitForSingleObject(m_hStop, 0); 
	}

	inline void ShutDown()
	{
		PostThreadMessage(m_dwThreadId, WM_QUIT, 0, 0);
	}

	bool SendOpCode(SOCKET hSocket, const COpCode &stOpCode, CEmClient* pClient);

	void ProcessSocketsTimeout();

public:
	__declspec(property(get=_GetSocketsProcessor))	CTaskProcessor_Sockets&	Sockets;
	__declspec(property(get=_GetDbProcessor))		CTaskProcessor_DB&		DB;
	__declspec(property(get=_GetFilesProcessor))	CTaskProcessor_Files&	Files;
	__declspec(property(get=_GetPreferences))		CPreferences&			Prefs;
	__declspec(property(get=_GetClientList))		CClientList&			ClientList;
	__declspec(property(get=_GetServerState))		ServerState_Struct&		ServerState;

	CTaskProcessor_Sockets&	_GetSocketsProcessor()	const { if(!m_pSocketsProcessor)throw;return *m_pSocketsProcessor; }
	CTaskProcessor_DB& 		_GetDbProcessor()		const { if(!m_pDbProcessor) throw;	return *m_pDbProcessor; }
	CTaskProcessor_Files& 	_GetFilesProcessor()	const { if(!m_pFilesProcessor) throw;	return *m_pFilesProcessor; }
	CPreferences& 			_GetPreferences()		const { if(!m_pPreferences) throw;	return *m_pPreferences; }
	CClientList&			_GetClientList()		const { if(!m_pClientList) throw;		return *m_pClientList; }
	ServerState_Struct&		_GetServerState()		{ return m_stServer; }

private:
	// Task processors
	CTaskProcessor_Main*	m_pMainProcessor;
	CTaskProcessor_Sockets*	m_pSocketsProcessor;
	CTaskProcessor_Files*	m_pFilesProcessor;
	CTaskProcessor_Logger*	m_pLoggerProcessor;
	CTaskProcessor_DB*		m_pDbProcessor;

	CClientList*	m_pClientList;

	CPreferences*	m_pPreferences;

	// Low-level classes might have want to access our private vars
	friend class CLoggable;
	friend class COpCode;
	friend struct CTask_LogOpcode;

	ServerState_Struct	m_stServer;

	DWORD	m_dwThreadId;
};

#endif // !defined(AFX_EMENGINE_H__F77B635A_251D_4554_9D96_DACFCA968759__INCLUDED_)
