// InfoListCtrl.h: interface for the CInfoListCtrl class.
// (p) 2002 by FoRcHa (a.k.a. NO)  [seppforcher38@hotmail.com]
//////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "COPTIONTREE\OptionTree.h"
#include "types.h"
#include "updownclient.h"

enum EnumInfoListType
{
	INFOLISTTYPE_NONE = 0,
	INFOLISTTYPE_SOURCE = 1,
	INFOLISTTYPE_FILE = 2
};

class CInfoListCtrl : public COptionTree
{
public:
	CInfoListCtrl();
	virtual ~CInfoListCtrl();
	void Localize();

protected:
	void ClearList();
	void CreateList();

	void FillSourcenameList();
	void UpdateData(bool bExpand = false);
	void SaveState(EnumInfoListType eListType)
	{
		if (eListType == INFOLISTTYPE_SOURCE && m_pClient != NULL)
		{
			if (UserData.General)
				m_bExpandedUser[0] = UserData.General->IsExpanded();
			if (UserData.Transfer)
				m_bExpandedUser[1] = UserData.Transfer->IsExpanded();
			if (UserData.Scores)
				m_bExpandedUser[2] = UserData.Scores->IsExpanded();
			if (UserData.RemoteScores)
				m_bExpandedUser[3] = UserData.RemoteScores->IsExpanded();
		}
		else if (eListType == INFOLISTTYPE_FILE && m_pPartFile != NULL)
		{
			if (FileData.General)
				m_bExpandedFile[0] = FileData.General->IsExpanded();
			if (FileData.Transfer)
				m_bExpandedFile[1] = FileData.Transfer->IsExpanded();
		}
	}

protected:
	CPartFile* 	m_pPartFile;
	CUpDownClient* m_pClient;
	EnumInfoListType m_eListType;
	
	UINT m_nUpdateTimer;
	COLORREF m_crEntryTextColor;

	bool m_bShown;
	bool m_bExpandedUser[4];
	bool m_bExpandedFile[2];
	
	struct _TAG_USERDATA_
	{
		COptionTreeItem			*General;
		COptionTreeItemStatic	*UserName;
		COptionTreeItemStatic	*UserHash;
		COptionTreeItemStatic	*ClientSoftware;
		COptionTreeItemStatic	*IPAddress;
		COptionTreeItemStatic	*ID;
		COptionTreeItemStatic	*ServerIP;
		COptionTreeItemStatic	*ServerName;

		COptionTreeItem			*Transfer;
		COptionTreeItemStatic	*CurDownloading;
		COptionTreeItemStatic	*DownloadedSession;
		COptionTreeItemStatic	*UploadedSession;
		COptionTreeItemStatic	*AverageDownloadrate;
		COptionTreeItemStatic	*AverageUploadrate;
		COptionTreeItemStatic	*DownloadedTotal;
		COptionTreeItemStatic	*UploadedTotal;

		COptionTreeItem			*Scores;
		COptionTreeItemStatic	*DlUpModifier;
		COptionTreeItemStatic	*CommunityUser;
		COptionTreeItemStatic	*Rating;
		COptionTreeItemStatic	*UploadQueueScore;

		COptionTreeItemStatic	*SFRatio;
		COptionTreeItemStatic	*RFRatio;

		COptionTreeItem			*RemoteScores;
		COptionTreeItemStatic	*RemoteDlUpModifier;
		COptionTreeItemStatic	*RemoteRating;
		COptionTreeItemStatic	*RemoteQueueRank;
	} UserData;
	
	struct _TAG_FILEDATA_
	{
		COptionTreeItem			*General;
		COptionTreeItemStatic	*FullName;
		COptionTreeItemStatic	*MetFile;
		COptionTreeItemStatic	*Hash;
		COptionTreeItemStatic	*FileSize;
		COptionTreeItemStatic	*RealSize;
		COptionTreeItemStatic	*PartFileStatus;
		COptionTreeItemComboBox	*SourceNames;
		bool					bSourceNameCombo;
		int						iSourceNameUpdateDelay;
		
		COptionTreeItem			*Transfer;
		COptionTreeItemStatic	*FoundSources;
		COptionTreeItemStatic	*FilepartCount;
		COptionTreeItemStatic	*PartAvailable;
		COptionTreeItemStatic	*Transferred;
		COptionTreeItemStatic	*DataRate;
		COptionTreeItemStatic	*Transferring;
		COptionTreeItemStatic	*CompletedSize;
		COptionTreeItemStatic	*LastSeenComplete;
		COptionTreeItemStatic	*LastProgress;
	} FileData;

public:
	BOOL Create(DWORD dwStyle, RECT rcRect, CWnd* pParentWnd, UINT nID);

	void SetList(EnumInfoListType enumItemType, void * pObject = NULL);

	EnumInfoListType GetType() { return m_eListType; }
	void SetType(EnumInfoListType in_ListType) { m_eListType = in_ListType; }

	CString GetName()
	{
		if (m_eListType == INFOLISTTYPE_SOURCE)
			return m_pClient == NULL ? _T("") : m_pClient->GetUserName();
		else
			return m_pPartFile == NULL ? _T("") : m_pPartFile->GetFileName();
	}

	CPartFile* GetFile() { return m_pPartFile; }
	CUpDownClient* GetClient() { return m_pClient; }

protected:
	//{{AFX_MSG(CInfoListCtrl)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnShowWindow(BOOL bShow, UINT nStatus);
};
