// Ini2.cpp: Ini-File-Interface by bond006 <rene.landgrebe@gmx.de>
// ****************************************************************
// This new interface replaces the old interface which was too slow

#include "stdafx.h"
#include "ini2.h"
#include "otherfunctions.h"

CIni::CIni(const CString &strFileName) : m_strDefCategory(_T(""))
{
	OpenFile(strFileName);
	ReadData();
}

CIni::~CIni()
{
	CloseFile();
}

void CIni::OpenFile(const CString& strFileName)
{
	if(m_hFile==CFile::hFileNull)
		BOOL bRetCode = Open(strFileName, CFile::modeCreate | CFile::modeNoTruncate | CFile::modeReadWrite | CFile::shareDenyWrite);
}

void CIni::CloseFile()
{
	if(m_hFile!=CFile::hFileNull)
		Close();
}

void CIni::ReadData()
{
	if(m_hFile!=CFile::hFileNull)
	{
		char tempBuffer[131072]; // change this value to support larger files than 128KB
		memzero(tempBuffer, sizeof(tempBuffer));
		Read(tempBuffer,sizeof(tempBuffer));
		m_strFileBuffer = tempBuffer;

		// Cut off blank lines at the end of the file
		m_strFileBuffer.TrimRight(_T("\r\n"));
	}
}

void CIni::SaveAndClose()
{
	if(m_hFile!=CFile::hFileNull)
	{
		SetLength(0);
		m_strFileBuffer.TrimRight(_T("\r\n"));
		Write(m_strFileBuffer,m_strFileBuffer.GetLength());
	}

	CloseFile();
}

void CIni::AddCategory(const CString& strCategoryName)
{
	if(!CategoryExist(strCategoryName))
	{
		if(m_strFileBuffer.GetLength()!=0)
			m_strFileBuffer += _T("\r\n");

		m_strFileBuffer += _T('[');
		m_strFileBuffer += strCategoryName;
		m_strFileBuffer += _T(']');
	}
}

void CIni::SetDefaultCategory(const CString& strCategoryName)
{
	AddCategory(strCategoryName);

	m_strDefCategory = strCategoryName;
}

void CIni::SetDefaultCategory(const TCHAR *pcCategoryName)
{
	m_strDefCategory = pcCategoryName;

	AddCategory(m_strDefCategory);
}

void CIni::DeleteCategory(const CString& strCategoryName)
{
	if(CategoryExist(strCategoryName))
	{
		CString catSearchString = _T("[");
		catSearchString += strCategoryName;
		catSearchString += _T(']');

		int nCatFirstCharPos = m_strFileBuffer.Find(catSearchString);

		// Check if the category is valid
		if(nCatFirstCharPos!=0)
		{
			catSearchString = _T("\r\n[");
			catSearchString += strCategoryName;
			catSearchString += _T(']');

			nCatFirstCharPos -= 2;
			nCatFirstCharPos = m_strFileBuffer.Find(catSearchString,nCatFirstCharPos);
			nCatFirstCharPos += 2;
		}

		int nCatLastCharPos = m_strFileBuffer.Find(_T("\r\n["),(nCatFirstCharPos + strCategoryName.GetLength() + 1));

		if(nCatLastCharPos == (-1))
			nCatLastCharPos = (m_strFileBuffer.GetLength());

		m_strFileBuffer.Delete(nCatFirstCharPos,(nCatLastCharPos - nCatFirstCharPos));
	}
}

void CIni::DeleteEntry(const CString& strCategoryName, const CString &strEntryName)
{
	if(EntryExist(strCategoryName,strEntryName))
	{
		CString catSearchString = _T("[");
		catSearchString += strCategoryName;
		catSearchString += _T(']');

		int nEntryFirstCharPos = 0;
		int nEntryLastCharPos = 0;
		int nCatFirstCharPos = 0;
		nCatFirstCharPos = m_strFileBuffer.Find(catSearchString);

		// Check if the category is valid
		if(nCatFirstCharPos!=0)
		{
			catSearchString = _T("\r\n[");
			catSearchString += strCategoryName;
			catSearchString += _T(']');

			nCatFirstCharPos -= 2;
			nCatFirstCharPos = m_strFileBuffer.Find(catSearchString,nCatFirstCharPos);
			nCatFirstCharPos += 2;
		}

		CString entrySearchString = _T("\r\n");
		entrySearchString += strEntryName;
		entrySearchString += _T('=');

		nEntryFirstCharPos = m_strFileBuffer.Find(entrySearchString,(nCatFirstCharPos + strCategoryName.GetLength() + 1));
		nEntryFirstCharPos += 2;

		nEntryLastCharPos = m_strFileBuffer.Find(_T("\r\n"),nEntryFirstCharPos);

		if(nEntryLastCharPos==(-1))
			nEntryLastCharPos = m_strFileBuffer.GetLength();
		else
			nEntryLastCharPos += 2; // remove entry including "\r\n"

		m_strFileBuffer.Delete(nEntryFirstCharPos,(nEntryLastCharPos - nEntryFirstCharPos));
	}
}

CString CIni::GetString(const TCHAR *pcEntryName, const CString& defValue)
{
	CString strTemp = GetValue(pcEntryName);

	if (strTemp.IsEmpty() && !EntryExist(m_strDefCategory, pcEntryName))
		return defValue;
	else
		return strTemp;
}

double CIni::GetDouble(const TCHAR *pcEntryName, double defValue)
{
	CString strTemp = GetValue(pcEntryName);

	if (strTemp.IsEmpty() && !EntryExist(m_strDefCategory, pcEntryName))
		return defValue;
	else
		return _tstof(strTemp);
}

float CIni::GetFloat(const TCHAR *pcEntryName, float defValue)
{
	CString strTemp = GetValue(pcEntryName);

	if (strTemp.IsEmpty() && !EntryExist(m_strDefCategory, pcEntryName))
		return defValue;
	else
		return (float)_tstof(strTemp);
}

int CIni::GetInt(const TCHAR *pcEntryName, int defValue)
{
	CString strTemp = GetValue(pcEntryName);

	if (strTemp.IsEmpty() && !EntryExist(m_strDefCategory, pcEntryName))
		return defValue;
	else
		return _tstoi(strTemp);
}

uint32 CIni::GetUInt32(const TCHAR *pcEntryName, uint32 defValue)
{
	CString strTemp = GetValue(pcEntryName);

	if (strTemp.IsEmpty() && !EntryExist(m_strDefCategory, pcEntryName))
		return defValue;
	else
		return _tstoi(strTemp);
}

uint64 CIni::GetUInt64(const TCHAR *pcEntryName, uint64 defValue)
{
	CString strTemp = GetValue(pcEntryName);

	if (strTemp.IsEmpty() && !EntryExist(m_strDefCategory, pcEntryName))
		return defValue;
	else
		return _tstoi64(strTemp);
}

WORD CIni::GetWORD(const TCHAR *pcEntryName, WORD defValue)
{
	return static_cast<WORD>(GetUInt32(pcEntryName, defValue));
}

bool CIni::GetBool(const TCHAR *pcEntryName, bool defValue)
{
	CString strTemp = GetValue(pcEntryName);

	if (strTemp.IsEmpty() && !EntryExist(m_strDefCategory, pcEntryName))
		return defValue;
	else
		return (_tstoi(strTemp) != 0);
}

void CIni::SetString(const TCHAR *pcEntryName, const CString& strValue)
{
	SetValue(pcEntryName, strValue);
}

void CIni::SetDouble(const TCHAR *pcEntryName, double dValue)
{
	CString strValue;
	strValue.Format(_T("%g"),dValue);

	SetValue(pcEntryName, strValue);
}

void CIni::SetFloat(const TCHAR *pcEntryName, float fValue)
{
	CString strValue;
	strValue.Format(_T("%g"),fValue);

	SetValue(pcEntryName, strValue);
}

void CIni::SetInt(const TCHAR *pcEntryName, int nValue)
{
	CString strValue;
	strValue.Format(_T("%d"),nValue);

	SetValue(pcEntryName, strValue);
}

void CIni::SetUInt32(const TCHAR *pcEntryName, uint32 nValue)
{
	CString strValue;
	strValue.Format(_T("%u"), nValue);

	SetValue(pcEntryName, strValue);
}

void CIni::SetUInt64(const TCHAR *pcEntryName, uint64 nValue)
{
	CString strValue;
	strValue.Format(_T("%I64u"),nValue);

	SetValue(pcEntryName, strValue);
}

void CIni::SetWORD(const TCHAR *pcEntryName, WORD wValue)
{
	SetUInt32(pcEntryName, wValue);
}

void CIni::SetBool(const TCHAR *pcEntryName, bool bValue)
{
	CString strValue = (bValue) ? _T("1") : _T("0");

	SetValue(pcEntryName, strValue);
}

void CIni::GetArray(CString* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp = GetString(pcEntryName, _T(""));

	int nPos = 0;
	int nLastPos = 0;
	int nEscapePos = 0;

	// bugfix: using escape characters for "\" and "," in strings - 08/25/03
	if(temp.Find(_T('\\')) != (-1))
	{
		for(int i=0;i<nElements;i++)
		{
			nLastPos = nPos;
			nPos = temp.Find(_T(','), nLastPos);
			nEscapePos = temp.Find(_T('\\'), nLastPos);

			if(nPos==(-2))
				pToArray[i] = _T("");

			if(nEscapePos==(-1) && nPos!=(-2))
			{
				if(nPos == (-1))
				{
					pToArray[i] = temp.Mid(nLastPos,(temp.GetLength() - nLastPos));
					nPos = -2;
				}
				else
				{
					pToArray[i] = (nPos >= 0) ? temp.Mid(nLastPos,(nPos - nLastPos)) : _T("");
					nPos++;
				}
			}
			else if(nEscapePos!=(-1) && nPos!=(-2))
			{
				if(nPos < nEscapePos)
				{
					if(nPos == (-1))
					{
						pToArray[i] = temp.Mid(nLastPos,(temp.GetLength() - nLastPos));
						nPos = -2;
					}
					else
					{
						pToArray[i] = (nPos >= 0) ? temp.Mid(nLastPos,(nPos - nLastPos)) : _T("");
						nPos++;
					}
				}
				else
				{
					int saveBegPos = nLastPos;

					while(nPos > nEscapePos && nEscapePos != (-1))
					{
						nLastPos = nEscapePos + 2;
						nPos = temp.Find(_T(','), nLastPos);
						nEscapePos = temp.Find(_T('\\'), nLastPos);
					}

					if(nPos == (-1))
					{
						pToArray[i] = temp.Mid(saveBegPos,(temp.GetLength() - saveBegPos));
						nPos = -2;
					}
					else
					{
						pToArray[i] = (nPos >= 0) ? temp.Mid(saveBegPos,(nPos - saveBegPos)) : _T("");
						nPos++;
					}
				}
			}

			pToArray[i].Replace(_T("\\,"),_T(","));
			pToArray[i].Replace(_T("\\\\"),_T("\\"));
		}
	}
	else
	{
		for(int i=0;i<nElements;i++)
			pToArray[i] = (nPos >= 0) ? temp.Tokenize(_T(","),nPos) : _T("");
	}
}

void CIni::GetArray(double* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp = GetString(pcEntryName, _T(""));

	int nPos = 0;

	for(int i=0;i<nElements;i++)
		pToArray[i] = (nPos >= 0) ? _tstof(temp.Tokenize(_T(","),nPos)) : 0.;
}

void CIni::GetArray(float* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp = GetString(pcEntryName, _T(""));

	int nPos = 0;

	for(int i=0;i<nElements;i++)
		pToArray[i] = (nPos >= 0) ? (float)_tstof(temp.Tokenize(_T(","),nPos)) : 0.f;
}

void CIni::GetArray(int* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp = GetString(pcEntryName, _T(""));

	int nPos = 0;

	for(int i=0;i<nElements;i++)
		pToArray[i] = (nPos >= 0) ? _tstoi(temp.Tokenize(_T(","),nPos)) : 0;
}

void CIni::GetArray(WORD* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp = GetString(pcEntryName, _T(""));

	int nPos = 0;

	for(int i=0;i<nElements;i++)
		pToArray[i] = (nPos >= 0) ? (WORD)_tstoi(temp.Tokenize(_T(","),nPos)) : 0;
}

void CIni::GetArray(bool* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp = GetString(pcEntryName, _T(""));

	int token, nPos = 0;

	for(int i=0;i<nElements;i++)
	{
		token = (nPos >= 0) ? _tstoi(temp.Tokenize(_T(","),nPos)) : 0;
		pToArray[i] = (token != 0);
	}
}

void CIni::SetArray(CString* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp;

	for(int i=0;i<nElements;i++)
	{
		// bugfix: using escape characters for "\" and "," in strings - 08/25/03
		pToArray[i].Replace(_T("\\"),_T("\\\\"));
		pToArray[i].Replace(_T(","),_T("\\,"));

		temp += pToArray[i];
		temp += _T(",");
	}

	temp.TrimRight(_T(','));

	SetString(pcEntryName, temp);
}

void CIni::SetArray(double* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp;

	for (int i = 0; i < nElements; i++)
		temp.AppendFormat(_T("%g,"), pToArray[i]);

	temp.TrimRight(_T(','));

	SetString(pcEntryName, temp);
}

void CIni::SetArray(float* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp;

	for (int i = 0; i < nElements; i++)
		temp.AppendFormat(_T("%g,"), pToArray[i]);

	temp.TrimRight(_T(','));

	SetString(pcEntryName, temp);
}

void CIni::SetArray(int* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp;

	for (int i = 0; i < nElements; i++)
		temp.AppendFormat(_T("%d,"), pToArray[i]);

	temp.TrimRight(_T(','));

	SetString(pcEntryName, temp);
}

void CIni::SetArray(WORD* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp;

	for (int i = 0; i < nElements; i++)
		temp.AppendFormat(_T("%u,"), pToArray[i]);

	temp.TrimRight(_T(','));

	SetString(pcEntryName, temp);
}

void CIni::SetArray(bool* pToArray, int nElements, const TCHAR *pcEntryName)
{
	CString temp;

	for (int i = 0; i < nElements; i++)
		temp.AppendFormat(_T("%d,"), pToArray[i]);

	temp.TrimRight(_T(','));

	SetString(pcEntryName, temp);
}

void CIni::SetValue(const TCHAR *pcEntryName, const CString &strValue)
{
	CString	strEntryName = pcEntryName;

	if (EntryExist(m_strDefCategory, pcEntryName))
	{
		CString catSearchString = _T("[");
		catSearchString += m_strDefCategory;
		catSearchString += _T(']');

		int nValueFirstCharPos = 0;
		int nEntryFirstCharPos = 0;
		int nEntryLastCharPos = 0;
		int nCatFirstCharPos = 0;

		nCatFirstCharPos = m_strFileBuffer.Find(catSearchString);

		// Check if the category is valid
		if(nCatFirstCharPos!=0)
		{
			catSearchString = _T("\r\n[");
			catSearchString += m_strDefCategory;
			catSearchString += _T(']');

			nCatFirstCharPos -= 2;
			nCatFirstCharPos = m_strFileBuffer.Find(catSearchString,nCatFirstCharPos);
			nCatFirstCharPos += 2;
		}

		CString entrySearchString = _T("\r\n");
		entrySearchString += strEntryName;
		entrySearchString += _T('=');

		nEntryFirstCharPos = m_strFileBuffer.Find(entrySearchString,(nCatFirstCharPos + m_strDefCategory.GetLength() + 1));
		nEntryFirstCharPos += 2;
		nEntryLastCharPos = m_strFileBuffer.Find(_T("\r\n"),nEntryFirstCharPos);

		if(nEntryLastCharPos==(-1))
			nEntryLastCharPos = m_strFileBuffer.GetLength();

		nValueFirstCharPos = nEntryFirstCharPos + strEntryName.GetLength() + 1;

		// Delete an existing value and insert a new value
		m_strFileBuffer.Delete(nValueFirstCharPos,(nEntryLastCharPos - nValueFirstCharPos));
		m_strFileBuffer.Insert(nValueFirstCharPos,strValue);
	}
	else
	{
		if(CategoryExist(m_strDefCategory))
		{
			CString catSearchString = _T("[");
			catSearchString += m_strDefCategory;
			catSearchString += _T(']');

			int nCatFirstCharPos = 0;
			int nCatLastCharPos = 0;

			nCatFirstCharPos = m_strFileBuffer.Find(catSearchString);

			// Check if the category is valid
			if(nCatFirstCharPos!=0)
			{
				catSearchString = _T("\r\n[");
				catSearchString += m_strDefCategory;
				catSearchString += _T(']');

				nCatFirstCharPos -= 2;
				nCatFirstCharPos = m_strFileBuffer.Find(catSearchString,nCatFirstCharPos);
				nCatFirstCharPos += 2;
			}

			nCatLastCharPos = m_strFileBuffer.Find(_T("\r\n["),(nCatFirstCharPos + m_strDefCategory.GetLength() + 1));

			CString entryString;

			strEntryName += _T('=');
			strEntryName += strValue;

			if(nCatLastCharPos==(-1))
			{
				nCatLastCharPos = m_strFileBuffer.GetLength();
				entryString = _T("\r\n");
				entryString += strEntryName;
			}
			else
			{
				nCatLastCharPos += 2;
				entryString = strEntryName;
				entryString += _T("\r\n");
			}

			m_strFileBuffer.Insert(nCatLastCharPos,entryString);
		}
	}
}

CString CIni::GetValue(const TCHAR *pcEntryName)
{
	CString	strEntryName = pcEntryName;

	if (ValueExist(m_strDefCategory, strEntryName))
	{
		CString catSearchString = _T("[");
		catSearchString += m_strDefCategory;
		catSearchString += _T(']');

		int nValueFirstCharPos = 0;
		int nEntryFirstCharPos = 0;
		int nEntryLastCharPos = 0;
		int nCatFirstCharPos = 0;

		nCatFirstCharPos = m_strFileBuffer.Find(catSearchString);

		// Check if the category is valid
		if(nCatFirstCharPos!=0)
		{
			catSearchString = _T("\r\n[");
			catSearchString += m_strDefCategory;
			catSearchString += _T(']');

			nCatFirstCharPos -= 2;
			nCatFirstCharPos = m_strFileBuffer.Find(catSearchString,nCatFirstCharPos);
			nCatFirstCharPos += 2;
		}

		CString entrySearchString = _T("\r\n");
		entrySearchString += strEntryName;
		entrySearchString += _T('=');

		nEntryFirstCharPos = m_strFileBuffer.Find(entrySearchString,(nCatFirstCharPos + m_strDefCategory.GetLength() + 1));
		nEntryFirstCharPos += 2;
		nEntryLastCharPos = m_strFileBuffer.Find(_T("\r\n"),nEntryFirstCharPos);

		if(nEntryLastCharPos==(-1))
			nEntryLastCharPos = m_strFileBuffer.GetLength();

		nValueFirstCharPos = nEntryFirstCharPos + strEntryName.GetLength() + 1;

		return m_strFileBuffer.Mid(nValueFirstCharPos,(nEntryLastCharPos - nValueFirstCharPos));
	}
	else
		return _T("");
}

bool CIni::CategoryExist(const CString& strCategoryName)
{
	CString catSearchString = _T("[");
	catSearchString += strCategoryName;
	catSearchString += _T(']');

	int nFindPos = m_strFileBuffer.Find(catSearchString);

	if(nFindPos==(-1))
		return false;

	// Check if the category is valid
	if(nFindPos!=0)
	{
		catSearchString = _T("\r\n[");
		catSearchString += strCategoryName;
		catSearchString += _T(']');

		nFindPos -= 2;
		nFindPos = m_strFileBuffer.Find(catSearchString,nFindPos);

		if(nFindPos==(-1))
			return false;
	}

	return true;
}

bool CIni::EntryExist(const CString &strCategoryName, const TCHAR *pcEntryName)
{
	CString catSearchString = _T("[");
	catSearchString += strCategoryName;
	catSearchString += _T(']');

	int nEntryFirstCharPos = 0;
	int nCatFirstCharPos = 0;
	int nCatLastCharPos = 0;
	nCatFirstCharPos = m_strFileBuffer.Find(catSearchString);

	if(nCatFirstCharPos==(-1))
		// Category does not exist
		return false;

	// Check if the category is valid
	if(nCatFirstCharPos!=0)
	{
		catSearchString = _T("\r\n[");
		catSearchString += strCategoryName;
		catSearchString += _T(']');

		nCatFirstCharPos -= 2;
		nCatFirstCharPos = m_strFileBuffer.Find(catSearchString,nCatFirstCharPos);

		if(nCatFirstCharPos==(-1))
			return false;

		nCatFirstCharPos += 2;
	}

	nCatLastCharPos = m_strFileBuffer.Find(_T("\r\n["),(nCatFirstCharPos + strCategoryName.GetLength() + 1));

	if(nCatLastCharPos==(-1))
		// No following category exists - end of category is end of file
		nCatLastCharPos = m_strFileBuffer.GetLength();
	else
		nCatLastCharPos += 2;

	CString entrySearchString = _T("\r\n");
	entrySearchString += pcEntryName;
	entrySearchString += _T('=');

	nEntryFirstCharPos = m_strFileBuffer.Find(entrySearchString,(nCatFirstCharPos + strCategoryName.GetLength() + 1));
	
	if(nEntryFirstCharPos!=(-1))
		nEntryFirstCharPos += 2;
	else
		return false;

	if(nEntryFirstCharPos<nCatFirstCharPos || nEntryFirstCharPos>nCatLastCharPos)
		// The entry we're searching for does not exist in the given category
		return false;

	return true;
}

bool CIni::ValueExist(const CString& strCategoryName, const CString& strEntryName)
{
	CString catSearchString = _T("[");
	catSearchString += strCategoryName;
	catSearchString += _T(']');

	int nEntryFirstCharPos = 0;
	int nCatFirstCharPos = 0;
	int nCatLastCharPos = 0;
	nCatFirstCharPos = m_strFileBuffer.Find(catSearchString);

	if(nCatFirstCharPos==(-1))
		// Category does not exist
		return false;

	// Check if the category is valid
	if(nCatFirstCharPos!=0)
	{
		catSearchString = _T("\r\n[");
		catSearchString += strCategoryName;
		catSearchString += _T(']');

		nCatFirstCharPos -= 2;
		nCatFirstCharPos = m_strFileBuffer.Find(catSearchString,nCatFirstCharPos);

		if(nCatFirstCharPos==(-1))
			return false;

		nCatFirstCharPos += 2;
	}

	nCatLastCharPos = m_strFileBuffer.Find(_T("\r\n["),(nCatFirstCharPos + strCategoryName.GetLength() + 1));

	if(nCatLastCharPos==(-1))
		// No following category exists - end of category is end of file
		nCatLastCharPos = m_strFileBuffer.GetLength();
	else
		nCatLastCharPos += 2;

	CString entrySearchString = _T("\r\n");
	entrySearchString += strEntryName;
	entrySearchString += _T('=');

	nEntryFirstCharPos = m_strFileBuffer.Find(entrySearchString,(nCatFirstCharPos + strCategoryName.GetLength() + 1));

	if(nEntryFirstCharPos!=(-1))
		nEntryFirstCharPos += 2;
	else
		return false;

	if(nEntryFirstCharPos<nCatFirstCharPos || nEntryFirstCharPos>nCatLastCharPos)
		// The entry we're searching for does not exist in the given category
		return false;

	int nEndOfLine = m_strFileBuffer.Find(_T("\r\n"),nEntryFirstCharPos);

	if(nEndOfLine==(-1))
		nEndOfLine = m_strFileBuffer.GetLength();

	if((nEntryFirstCharPos + strEntryName.GetLength() + 1) == nEndOfLine)
		return false;

	return true;
}
