//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Client to Server communication

#pragma once

#include "types.h"
#include "server.h"
#include "packets.h"
#include "ServerListCtrl.h"
#include "Preferences.h"
#include "server.h"
#include "serverlist.h"
#include "preferences.h"
#include "serversocket.h"
#include "Loggable.h"

#define CS_RETRYCONNECTTIME  30 // seconds

#ifdef OLD_SOCKETS_ENABLED

class CUDPSocket;

class CServerConnect : public CLoggable{
public:
					CServerConnect(CServerList* in_serverlist, CPreferences* in_prefs);
					~CServerConnect();
	void			ConnectionFailed(CServerSocket* sender);
	void			ConnectionEstablished(CServerSocket* sender);

	void			ConnectToAnyServer(uint32 iStartIndex = 0xFFFFFFFF, bool bPrioritySort=true, bool bIsAuto=true);
	void			ConnectToServer(CServer &server);
	void			StopConnectionTry();
	static			VOID CALLBACK RetryConnectCallback(HWND hWnd, UINT nMsg, UINT nId, DWORD dwTime);
	static			VOID CALLBACK CheckInternetCallback(HWND hWnd, UINT nMsg, UINT nId, DWORD dwTime);

	void			CheckForTimeout();
	void			DestroySocket(CServerSocket* pSck);	// safe socket closure and destruction
	bool			SendPacket(Packet *pPacket,bool bDelPacket = true, CServerSocket *pServerSocket = NULL);
	bool			SendUDPPacket(Packet* packet,CServer* host, bool delpacket = false );
	bool			Disconnect();
	bool			IsConnecting() const		{return m_bConnecting;}
	bool			IsConnected() const			{return m_bConnected;}
	bool			IsICCActive() const			{return ((m_iCheckTimerID != 0) || (m_hICCThread != 0));}
	uint32			GetClientID() const			{return m_dwClientID;}
	CServer*		GetCurrentServer();
	CServer*		GetConnectingServer();	// Server status in WebServer
	bool			IsLowID()					{return (m_dwClientID < 0x1000000);}
	void			SetClientID(uint32 newid);
	bool			IsLocalServer(uint32 dwIP, uint16 nPort);
	void			TryAnotherConnectionRequest();
	void			InitLocalIP();
	uint32			GetLocalIP()				{return m_dwLocalIP;}
	void			KeepConnectionAlive();

private:
	bool			IsAliveURL(LPCTSTR sURL);
	static void		IsAliveURLThread(void *);

private:
	bool			m_bConnecting;
	bool			m_bConnected;
	short			m_iNumConnectionAttempts;
	CServer		   *m_pConnectedServerSocket;
	uint32			m_iLastServerListStartPos;
	CPreferences   *m_pPrefs;
	CServerSocket  *m_pConnectedSocket;
	CServerList	   *m_pUsedServerList;
	CUDPSocket	   *m_pUDPSocket;
	UINT			m_iRetryTimerID;
	UINT			m_iCheckTimerID;
	uint32			m_dwLocalIP;
	uint32			m_dwClientID;
	HANDLE			m_hICCThread;

	CTypedPtrList<CPtrList, CServerSocket *> m_openSocketList;	// list of currently opened sockets
	CMap<ULONG ,ULONG&,CServerSocket*,CServerSocket*> m_mapConnectionAttempts;	// Map of connect times to server sockets
};

#endif //OLD_SOCKETS_ENABLED
